#define TK_ABORT                           1
#define TK_AFTER                           2
#define TK_AGG_FUNCTION                    3
#define TK_ALL                             4
#define TK_AND                             5
#define TK_AS                              6
#define TK_ASC                             7
#define TK_BEFORE                          8
#define TK_BEGIN                           9
#define TK_BETWEEN                        10
#define TK_BITAND                         11
#define TK_BITNOT                         12
#define TK_BITOR                          13
#define TK_BY                             14
#define TK_CASCADE                        15
#define TK_CASE                           16
#define TK_CHECK                          17
#define TK_CLUSTER                        18
#define TK_COLLATE                        19
#define TK_COLUMN                         20
#define TK_COMMA                          21
#define TK_COMMENT                        22
#define TK_COMMIT                         23
#define TK_CONCAT                         24
#define TK_CONFLICT                       25
#define TK_CONSTRAINT                     26
#define TK_COPY                           27
#define TK_CREATE                         28
#define TK_DEFAULT                        29
#define TK_DEFERRABLE                     30
#define TK_DEFERRED                       31
#define TK_DELETE                         32
#define TK_DELIMITERS                     33
#define TK_DESC                           34
#define TK_DISTINCT                       35
#define TK_DOT                            36
#define TK_DROP                           37
#define TK_EACH                           38
#define TK_ELSE                           39
#define TK_END                            40
#define TK_END_OF_FILE                    41
#define TK_EQ                             42
#define TK_EXCEPT                         43
#define TK_EXPLAIN                        44
#define TK_FAIL                           45
#define TK_FLOAT                          46
#define TK_FOR                            47
#define TK_FOREIGN                        48
#define TK_FROM                           49
#define TK_FUNCTION                       50
#define TK_GE                             51
#define TK_GLOB                           52
#define TK_GROUP                          53
#define TK_GT                             54
#define TK_HAVING                         55
#define TK_ID                             56
#define TK_IGNORE                         57
#define TK_ILLEGAL                        58
#define TK_IMMEDIATE                      59
#define TK_IN                             60
#define TK_INDEX                          61
#define TK_INITIALLY                      62
#define TK_INSERT                         63
#define TK_INSTEAD                        64
#define TK_INTEGER                        65
#define TK_INTERSECT                      66
#define TK_INTO                           67
#define TK_IS                             68
#define TK_ISNULL                         69
#define TK_JOIN                           70
#define TK_JOIN_KW                        71
#define TK_KEY                            72
#define TK_LE                             73
#define TK_LIKE                           74
#define TK_LIMIT                          75
#define TK_LP                             76
#define TK_LSHIFT                         77
#define TK_LT                             78
#define TK_MATCH                          79
#define TK_MINUS                          80
#define TK_NE                             81
#define TK_NOT                            82
#define TK_NOTNULL                        83
#define TK_NULL                           84
#define TK_OF                             85
#define TK_OFFSET                         86
#define TK_ON                             87
#define TK_OR                             88
#define TK_ORACLE_OUTER_JOIN              89
#define TK_ORDER                          90
#define TK_PLUS                           91
#define TK_PRAGMA                         92
#define TK_PRIMARY                        93
#define TK_RAISE                          94
#define TK_REFERENCES                     95
#define TK_REM                            96
#define TK_REPLACE                        97
#define TK_RESTRICT                       98
#define TK_ROLLBACK                       99
#define TK_ROW                            100
#define TK_RP                             101
#define TK_RSHIFT                         102
#define TK_SELECT                         103
#define TK_SEMI                           104
#define TK_SET                            105
#define TK_SLASH                          106
#define TK_SPACE                          107
#define TK_STAR                           108
#define TK_STATEMENT                      109
#define TK_STRING                         110
#define TK_TABLE                          111
#define TK_TEMP                           112
#define TK_THEN                           113
#define TK_TRANSACTION                    114
#define TK_TRIGGER                        115
#define TK_UMINUS                         116
#define TK_UNCLOSED_STRING                117
#define TK_UNION                          118
#define TK_UNIQUE                         119
#define TK_UPDATE                         120
#define TK_UPLUS                          121
#define TK_USING                          122
#define TK_VACUUM                         123
#define TK_VALUES                         124
#define TK_VIEW                           125
#define TK_WHEN                           126
#define TK_WHERE                          127
