/* Automatically generated file.  Do not edit */
#define OP_Goto                          1
#define OP_Gosub                         2
#define OP_Return                        3
#define OP_Halt                          4
#define OP_Integer                       5
#define OP_String                        6
#define OP_Pop                           7
#define OP_Dup                           8
#define OP_Pull                          9
#define OP_Push                         10
#define OP_ColumnName                   11
#define OP_Callback                     12
#define OP_NullCallback                 13
#define OP_Concat                       14
#define OP_Add                          15
#define OP_Subtract                     16
#define OP_Multiply                     17
#define OP_Divide                       18
#define OP_Remainder                    19
#define OP_Function                     20
#define OP_BitAnd                       21
#define OP_BitOr                        22
#define OP_ShiftLeft                    23
#define OP_ShiftRight                   24
#define OP_AddImm                       25
#define OP_MustBeInt                    26
#define OP_Eq                           27
#define OP_Ne                           28
#define OP_Lt                           29
#define OP_Le                           30
#define OP_Gt                           31
#define OP_Ge                           32
#define OP_StrEq                        33
#define OP_StrNe                        34
#define OP_StrLt                        35
#define OP_StrLe                        36
#define OP_StrGt                        37
#define OP_StrGe                        38
#define OP_And                          39
#define OP_Or                           40
#define OP_Negative                     41
#define OP_AbsValue                     42
#define OP_Not                          43
#define OP_BitNot                       44
#define OP_Noop                         45
#define OP_If                           46
#define OP_IfNot                        47
#define OP_IsNull                       48
#define OP_NotNull                      49
#define OP_MakeRecord                   50
#define OP_MakeIdxKey                   51
#define OP_MakeKey                      52
#define OP_IncrKey                      53
#define OP_Checkpoint                   54
#define OP_Transaction                  55
#define OP_Commit                       56
#define OP_Rollback                     57
#define OP_ReadCookie                   58
#define OP_SetCookie                    59
#define OP_VerifyCookie                 60
#define OP_OpenAux                      61
#define OP_OpenWrAux                    62
#define OP_OpenWrite                    63
#define OP_Open                         64
#define OP_OpenTemp                     65
#define OP_RenameCursor                 66
#define OP_Close                        67
#define OP_MoveLt                       68
#define OP_MoveTo                       69
#define OP_Distinct                     70
#define OP_NotFound                     71
#define OP_Found                        72
#define OP_IsUnique                     73
#define OP_NotExists                    74
#define OP_NewRecno                     75
#define OP_PutIntKey                    76
#define OP_PutStrKey                    77
#define OP_Delete                       78
#define OP_KeyAsData                    79
#define OP_Column                       80
#define OP_Recno                        81
#define OP_FullKey                      82
#define OP_NullRow                      83
#define OP_Last                         84
#define OP_Rewind                       85
#define OP_Prev                         86
#define OP_Next                         87
#define OP_IdxPut                       88
#define OP_IdxDelete                    89
#define OP_IdxRecno                     90
#define OP_IdxLT                        91
#define OP_IdxGT                        92
#define OP_IdxGE                        93
#define OP_Destroy                      94
#define OP_Clear                        95
#define OP_CreateIndex                  96
#define OP_CreateTable                  97
#define OP_IntegrityCk                  98
#define OP_ListWrite                    99
#define OP_ListRewind                  100
#define OP_ListRead                    101
#define OP_ListReset                   102
#define OP_ListPush                    103
#define OP_ListPop                     104
#define OP_SortPut                     105
#define OP_SortMakeRec                 106
#define OP_SortMakeKey                 107
#define OP_Sort                        108
#define OP_SortNext                    109
#define OP_SortCallback                110
#define OP_SortReset                   111
#define OP_FileOpen                    112
#define OP_FileRead                    113
#define OP_FileColumn                  114
#define OP_MemStore                    115
#define OP_MemLoad                     116
#define OP_MemIncr                     117
#define OP_AggReset                    118
#define OP_AggInit                     119
#define OP_AggFunc                     120
#define OP_AggFocus                    121
#define OP_AggSet                      122
#define OP_AggGet                      123
#define OP_AggNext                     124
#define OP_SetInsert                   125
#define OP_SetFound                    126
#define OP_SetNotFound                 127
#define OP_SetFirst                    128
#define OP_SetNext                     129
